"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const runtime_1 = require("@pajn/control-flow/cjs/runtime/runtime");
const apollo_boost_1 = require("apollo-boost");
const apollo_server_hapi_1 = require("apollo-server-hapi");
const graphql_tools_1 = require("graphql-tools");
const graphql_type_json_1 = require("graphql-type-json");
const Hapi = require("hapi");
const inert = require("inert");
const fetch = require("isomorphic-fetch");
const Joi = require("joi");
const node_fetch_1 = require("node-fetch");
const path_1 = require("path");
const raxa_common_1 = require("raxa-common");
const nodeImplementations_1 = require("./nodeImplementations");
const nodeTypes_1 = require("./nodeTypes");
const plugin_1 = require("./plugin");
const typeDefs = apollo_server_hapi_1.gql `
  scalar JSON

  type Program {
    id: String
    name: String
    graph: JSON
  }

  input ProgramInput {
    id: String
    name: String
    graph: JSON
  }

  type Query {
    program(id: String!): Program
    programs: [Program!]
  }

  type Mutation {
    saveProgram(program: ProgramInput!): Program
  }
`;
class ControlFlowPlugin extends raxa_common_1.Plugin {
    constructor() {
        super(...arguments);
        this.deviceStatusListeners = new Set();
        this.runtimes = new Map();
        this.getSettings = () => (this.state.getState().plugins[plugin_1.default.id].settings || {
            programs: {},
        });
        this.setSettings = (settings) => this.dispatch(raxa_common_1.actions.pluginUpdated, { plugin: { id: plugin_1.default.id, settings } });
    }
    async start() {
        const hapiServer = new Hapi.Server({
            port: 13001,
            debug: {
                log: ['*'],
                request: ['*'],
            },
            routes: {
                files: {
                    relativeTo: path_1.join(__dirname, '../ui/build'),
                },
            },
        });
        const raxaClient = new apollo_boost_1.default({
            uri: 'http://localhost:9000/graphql',
            fetchOptions: { fetch },
        });
        const nodeTypes = nodeTypes_1.getNodeTypes(raxaClient);
        const { nodeImplementations, httpReceive } = nodeImplementations_1.getNodeImplementations(raxaClient);
        Object.values(this.getSettings().programs).forEach((program) => {
            const runtime = runtime_1.createRuntime({
                program: {
                    graph: program.graph,
                    nodeTypes,
                },
                implementations: nodeImplementations(this),
            });
            runtime.start();
            this.runtimes.set(program.id, runtime);
        });
        const resolvers = {
            Query: {
                program: (_, { id }) => this.getSettings().programs[id],
                programs: () => this.getSettings().programs,
            },
            Mutation: {
                saveProgram: (_, { program }) => {
                    if (!program.id) {
                        program.id = Date.now().toString();
                    }
                    this.setSettings({
                        programs: Object.assign(Object.assign({}, this.getSettings().programs), { [program.id]: program }),
                    });
                    const runtime = this.runtimes.get(program.id);
                    if (runtime) {
                        runtime.updateProgram({
                            graph: this.getSettings().programs[1].graph,
                            nodeTypes,
                        });
                    }
                    else {
                        const runtime = runtime_1.createRuntime({
                            program: {
                                graph: program.graph,
                                nodeTypes,
                            },
                            implementations: nodeImplementations(this),
                        });
                        runtime.start();
                        this.runtimes.set(program.id, runtime);
                    }
                    return program;
                },
            },
        };
        const schema = graphql_tools_1.makeExecutableSchema({
            typeDefs,
            resolvers,
        });
        Object.assign(schema._typeMap.JSON, graphql_type_json_1.default);
        const apolloServer = new apollo_server_hapi_1.ApolloServer({
            schema,
        });
        await apolloServer.applyMiddleware({
            app: hapiServer,
            path: '/graphql',
            cors: true,
        });
        apolloServer.installSubscriptionHandlers(hapiServer.listener);
        hapiServer.route({
            method: '*',
            path: '/raxagraphql',
            async handler(request, h) {
                const response = await node_fetch_1.default(`http://127.0.0.1:9000/graphql${request.url.search}`, {
                    method: request.method.toUpperCase(),
                    headers: request.headers,
                    body: request.payload,
                    redirect: 'manual',
                });
                const responseObj = h
                    .response(await response.buffer())
                    .code(response.status);
                response.headers.forEach((value, key) => {
                    if (!['content-length', 'content-encoding', 'accept-ranges'].includes(key)) {
                        responseObj.header(key, value);
                    }
                });
                return responseObj;
            },
            options: {
                cors: true,
                payload: {
                    output: 'data',
                    parse: false,
                },
            },
        });
        await hapiServer.register(inert);
        hapiServer.route({
            method: ['GET', 'POST'],
            path: '/variable/{name}',
            handler: (request, h) => {
                const didExist = httpReceive(request.params.name, request.query.value);
                return didExist
                    ? h.response('ok').code(200)
                    : h.response('missing variable').code(400);
            },
            options: {
                validate: {
                    query: {
                        value: Joi.string().required(),
                    },
                },
            },
        });
        hapiServer.route({
            method: 'GET',
            path: '/{param*}',
            handler: {
                directory: {
                    path: path_1.join(__dirname, '../ui/build'),
                    index: true,
                    lookupCompressed: true,
                },
            },
        });
        // return index.html for everything else
        hapiServer.ext('onPreResponse', (request, h) => {
            const response = request.response;
            if (response && response.isBoom && response.output.statusCode === 404) {
                return h.file(path_1.join(__dirname, '../ui/build/index.html'), {
                    confine: false,
                });
            }
            return h.continue;
        });
        await hapiServer.start();
        this.server = hapiServer;
        this.log.info(`CF Server running at: ${hapiServer.info.uri}`);
    }
    async stop() {
        if (this.server) {
            await this.server.stop();
        }
    }
    onAction(action) {
        if (action.type === 'statusUpdated') {
            for (const listener of this.deviceStatusListeners) {
                listener(action.payload);
            }
        }
    }
}
exports.default = ControlFlowPlugin;
//# sourceMappingURL=index.js.map