"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNodeImplementations = void 0;
const logic_1 = require("@pajn/control-flow/cjs/nodes/logic");
const math_1 = require("@pajn/control-flow/cjs/nodes/math");
const utilities_1 = require("@pajn/control-flow/cjs/nodes/utilities");
const apollo_boost_1 = require("apollo-boost");
const checkNodeImplementations = (nodeImplementations) => nodeImplementations;
const getNodeImplementations = (raxaClient) => {
    const variableValues = new Map();
    const httpListeners = new Map();
    function httpReceive(name, value) {
        if (httpListeners.has(name)) {
            httpListeners.get(name)(value);
            return true;
        }
        else {
            return false;
        }
    }
    const callDevice = (call) => raxaClient.mutate({
        mutation: apollo_boost_1.gql `
        mutation(
          $deviceId: String!
          $interfaceId: String!
          $method: String!
          $arguments: JSON!
        ) {
          callDevice(
            deviceId: $deviceId
            interfaceId: $interfaceId
            method: $method
            arguments: $arguments
          ) {
            id
          }
        }
      `,
        variables: call,
    });
    const setDeviceStatus = (modification) => raxaClient.mutate({
        mutation: apollo_boost_1.gql `
        mutation(
          $deviceId: String!
          $interfaceId: String!
          $statusId: String!
          $value: JSON!
        ) {
          setDeviceStatus(
            deviceId: $deviceId
            interfaceId: $interfaceId
            statusId: $statusId
            value: $value
          ) {
            id
          }
        }
      `,
        variables: modification,
    });
    const raxaNodeImplementations = (plugin) => checkNodeImplementations({
        deviceStatusChanged: {
            id: 'deviceStatusChanged',
            onStart(ctrl) {
                ctrl.state = {
                    status: {},
                    listener: (modification) => {
                        if (modification.deviceId === ctrl.getProperty('device') &&
                            modification.interfaceId === ctrl.getProperty('interface')) {
                            ctrl.state.status[`status_${modification.statusId}`] =
                                modification.value;
                            ctrl.dispatchEvent('event');
                        }
                    },
                };
                plugin.deviceStatusListeners.add(ctrl.state.listener);
            },
            onStop(ctrl) {
                plugin.deviceStatusListeners.delete(ctrl.state.listener);
            },
            getPort(portId, ctrl) {
                return ctrl.state.status[portId];
            },
        },
        getDeviceStatus: {
            id: 'getDeviceStatus',
            getPort(portId, ctrl) {
                const deviceId = ctrl.getProperty('device');
                const interfaceId = ctrl.getProperty('interface');
                const statusId = portId.slice(7);
                const state = plugin.state.getState();
                return (state.status[deviceId] &&
                    state.status[deviceId][interfaceId] &&
                    state.status[deviceId][interfaceId][statusId]);
            },
        },
        setDeviceStatus: {
            id: 'setDeviceStatus',
            onEvent(_, ctrl) {
                setDeviceStatus({
                    deviceId: ctrl.getProperty('device'),
                    interfaceId: ctrl.getProperty('interface'),
                    statusId: ctrl.getProperty('status'),
                    value: ctrl.getPort('value'),
                });
            },
        },
        callDevice: {
            id: 'callDevice',
            onStart(ctrl) {
                ctrl.state = {};
            },
            onEvent(_, ctrl) {
                callDevice({
                    deviceId: ctrl.getProperty('device'),
                    interfaceId: ctrl.getProperty('interface'),
                    method: ctrl.getProperty('method'),
                    arguments: {},
                }).then(() => {
                    ctrl.state.didError = false;
                    ctrl.dispatchEvent('onDone');
                }, () => {
                    ctrl.state.didError = true;
                    ctrl.dispatchEvent('onDone');
                });
            },
            getPort(_, ctrl) {
                return ctrl.state.didError;
            },
        },
        eventReceived: {
            id: 'eventReceived',
            onStart(ctrl) {
                const iface = ctrl.getProperty('interface');
                const event = ctrl.getProperty('event');
                if (iface && event) {
                    ctrl.state = {
                        iface,
                        event,
                        dispose: plugin.listenOn(iface, event, eventObj => {
                            ctrl.state.eventObj = eventObj;
                            ctrl.dispatchEvent('event');
                        }),
                    };
                }
            },
            onStop(ctrl) {
                if (ctrl.state && ctrl.state.dispose) {
                    ctrl.state.dispose();
                }
            },
            getPort(_, ctrl) {
                return ctrl.state.eventObj;
            },
        },
        fireEvent: {
            id: 'fireEvent',
            onEvent(_, ctrl) {
                const iface = ctrl.getProperty('interface');
                const event = ctrl.getProperty('event');
                const eventObj = ctrl.getPort('eventObj'); // TODO: Replace with a check for connection
                if (iface && event && eventObj) {
                    plugin.fireEvent(iface, event, eventObj);
                }
            },
        },
        getVariable: {
            id: 'getVariable',
            getPort(_, ctrl) {
                return variableValues.get(ctrl.getProperty('name'));
            },
        },
        setVariable: {
            id: 'setVariable',
            onEvent(_, ctrl) {
                return variableValues.set(ctrl.getProperty('name'), ctrl.getPort('value'));
            },
        },
        httpRequest: {
            id: 'httpRequest',
            onStart(ctrl) {
                ctrl.state = {
                    value: undefined,
                    listener: (value) => {
                        ctrl.state.value = value;
                        ctrl.dispatchEvent('event');
                    },
                };
                httpListeners.set(ctrl.getProperty('name'), ctrl.state.listener);
            },
            onStop(ctrl) {
                httpListeners.delete(ctrl.getProperty('name'));
            },
            onPropertiesChanged(ctrl, oldProperties) {
                if (ctrl.getProperty('name') !== oldProperties.name) {
                    httpListeners.delete(oldProperties.name);
                    httpListeners.set(ctrl.getProperty('name'), ctrl.state.listener);
                }
            },
            getPort(_, ctrl) {
                return ctrl.state.value;
            },
        },
        switch: {
            id: 'switch',
            getPort: function getPort(_, ctrl) {
                if (ctrl.getPort('condition')) {
                    return ctrl.getPort('trueValue');
                }
                else {
                    return ctrl.getPort('falseValue');
                }
            },
        },
    });
    const nodeImplementations = (plugin) => (Object.assign(Object.assign(Object.assign(Object.assign({}, raxaNodeImplementations(plugin)), logic_1.implementations), math_1.implementations), utilities_1.implementations));
    return { nodeImplementations, httpReceive };
};
exports.getNodeImplementations = getNodeImplementations;
//# sourceMappingURL=nodeImplementations.js.map