"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNodeTypes = void 0;
const logic_1 = require("@pajn/control-flow/cjs/nodes/logic");
const math_1 = require("@pajn/control-flow/cjs/nodes/math");
const utilities_1 = require("@pajn/control-flow/cjs/nodes/utilities");
const apollo_boost_1 = require("apollo-boost");
const checkNodeTypes = (nodeTypes) => nodeTypes;
const getNodeTypes = (raxaClient) => {
    const getDevices = () => raxaClient
        .query({
        query: apollo_boost_1.gql `
          query {
            devices {
              id
              value: id
              name
            }
          }
        `,
    })
        .then(({ data }) => data.devices);
    const getAllInterfaces = () => raxaClient
        .query({
        query: apollo_boost_1.gql `
          query {
            interfaces {
              id
              value: id
              name
            }
          }
        `,
    })
        .then(({ data }) => data.interfaces);
    const getDeviceInterfaces = ({ device }) => device
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($device: String!) {
                device(id: $device) {
                  id
                  interfaces {
                    id
                    value: id
                    name
                  }
                }
              }
            `,
            variables: { device },
        })
            .then(({ data }) => data.device.interfaces)
        : [];
    const getStatuses = ({ device, interface: iface, }) => device && iface
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($iface: String!) {
                interface(id: $iface) {
                  id
                  status
                }
              }
            `,
            variables: { iface },
        })
            .then(({ data }) => Object.values(data.interface.status || {}).map(status => (Object.assign(Object.assign({}, status), { value: status.id, name: status.name || status.id }))))
        : [];
    const getMethods = ({ device, interface: iface, }) => device && iface
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($iface: String!) {
                interface(id: $iface) {
                  id
                  methods
                }
              }
            `,
            variables: { iface },
        })
            .then(({ data }) => Object.values(data.interface.methods || {}).map((methods) => ({
            id: methods.id,
            value: methods.id,
            name: methods.name || methods.id,
        })))
        : [];
    const getEvents = ({ interface: iface }) => iface
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($iface: String!) {
                interface(id: $iface) {
                  id
                  events
                }
              }
            `,
            variables: { iface },
        })
            .then(({ data }) => Object.values(data.interface.events || {}).map((events) => ({
            id: events.id,
            value: events.id,
            name: events.name || events.id,
        })))
        : [];
    const getStatus = ({ interface: iface, status, }) => iface && status
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($iface: String!) {
                interface(id: $iface) {
                  id
                  status
                }
              }
            `,
            variables: { iface },
        })
            .then(({ data }) => data.interface.status && data.interface.status[status])
        : undefined;
    const getMethod = ({ interface: iface, method, }) => iface && method
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($iface: String!) {
                interface(id: $iface) {
                  id
                  methods
                }
              }
            `,
            variables: { iface },
        })
            .then(({ data }) => data.interface.methods && data.interface.methods[method])
        : undefined;
    const getEvent = ({ interface: iface, event, }) => iface && event
        ? raxaClient
            .query({
            query: apollo_boost_1.gql `
              query($iface: String!) {
                interface(id: $iface) {
                  id
                  events
                }
              }
            `,
            variables: { iface },
        })
            .then(({ data }) => data.interface.events && data.interface.events[event])
        : undefined;
    const dataTypesRaxaToControlFlow = {
        string: 'String',
        number: 'Num',
        integer: 'Num',
        boolean: 'Bool',
        enum: 'Enum',
        object: 'Object',
    };
    const propertyToValuePort = (id, property, kind) => {
        let properties = undefined;
        if (property.type === 'object') {
            properties = {};
            for (const [id, prop] of Object.entries(property.properties)) {
                properties[id] = propertyToValuePort(id, prop);
            }
        }
        return {
            id,
            kind: kind,
            dataType: dataTypesRaxaToControlFlow[property.type],
            name: property.name || property.id,
            values: property.type === 'enum' ? property.values : undefined,
            properties,
        };
    };
    const raxaNodes = checkNodeTypes({
        deviceStatusChanged: {
            id: 'deviceStatusChanged',
            name: 'Device status changed',
            group: ['Raxa'],
            ports: {
                event: { id: 'event', kind: 'EventOutput' },
            },
            properties: {
                device: {
                    id: 'device',
                    name: 'Device',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDevices,
                },
                interface: {
                    id: 'interface',
                    name: 'Interface',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDeviceInterfaces,
                },
            },
            async dynamicOverrides(properties) {
                const statuses = await getStatuses(properties);
                const ports = Object.assign({}, this.ports);
                for (const status of statuses) {
                    if (dataTypesRaxaToControlFlow[status.type]) {
                        const id = `status_${status.id}`;
                        ports[id] = propertyToValuePort(id, status, 'ValueOutput');
                    }
                }
                return { ports };
            },
        },
        getDeviceStatus: {
            id: 'getDeviceStatus',
            name: 'Get device status',
            group: ['Raxa'],
            ports: {},
            properties: {
                device: {
                    id: 'device',
                    name: 'Device',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDevices,
                },
                interface: {
                    id: 'interface',
                    name: 'Interface',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDeviceInterfaces,
                },
            },
            async dynamicOverrides(properties) {
                const statuses = await getStatuses(properties);
                const ports = Object.assign({}, this.ports);
                for (const status of statuses) {
                    if (dataTypesRaxaToControlFlow[status.type]) {
                        const id = `status_${status.id}`;
                        ports[id] = propertyToValuePort(id, status, 'ValueOutput');
                    }
                }
                return { ports };
            },
        },
        setDeviceStatus: {
            id: 'setDeviceStatus',
            name: 'Set device status',
            group: ['Raxa'],
            ports: {
                event: { id: 'event', kind: 'EventInput' },
            },
            properties: {
                device: {
                    id: 'device',
                    name: 'Device',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDevices,
                },
                interface: {
                    id: 'interface',
                    name: 'Interface',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDeviceInterfaces,
                },
                status: {
                    id: 'status',
                    name: 'Status',
                    dataType: 'Enum',
                    required: true,
                    getValues: getStatuses,
                },
            },
            async dynamicOverrides(properties) {
                const status = await getStatus(properties);
                const ports = Object.assign({}, this.ports);
                if (status && dataTypesRaxaToControlFlow[status.type]) {
                    const id = `value`;
                    ports[id] = propertyToValuePort(id, status, 'ValueInput');
                }
                return { ports };
            },
        },
        callDevice: {
            id: 'callDevice',
            name: 'Call device',
            group: ['Raxa'],
            ports: {
                event: { id: 'event', kind: 'EventInput' },
                onDone: {
                    id: 'onDone',
                    name: 'on done',
                    kind: 'EventOutput',
                },
                didError: {
                    id: 'didError',
                    name: 'did error',
                    kind: 'ValueOutput',
                    dataType: 'Bool',
                },
            },
            properties: {
                device: {
                    id: 'device',
                    name: 'Device',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDevices,
                },
                interface: {
                    id: 'interface',
                    name: 'Interface',
                    dataType: 'Enum',
                    required: true,
                    getValues: getDeviceInterfaces,
                },
                method: {
                    id: 'method',
                    name: 'Method',
                    dataType: 'Enum',
                    required: true,
                    getValues: getMethods,
                },
            },
            async dynamicOverrides(properties) {
                const method = await getMethod(properties);
                const ports = Object.assign({}, this.ports);
                if (method) {
                    for (const arg of Object.values(method.arguments)) {
                        if (dataTypesRaxaToControlFlow[arg.type]) {
                            const id = `arg_${arg.id}`;
                            ports[id] = propertyToValuePort(id, arg, 'ValueInput');
                        }
                    }
                }
                return { ports };
            },
        },
        eventReceived: {
            id: 'eventReceived',
            name: 'Event received',
            group: ['Raxa'],
            ports: {
                event: {
                    id: 'event',
                    kind: 'EventOutput',
                },
            },
            properties: {
                interface: {
                    id: 'interface',
                    name: 'Interface',
                    dataType: 'Enum',
                    required: true,
                    getValues: getAllInterfaces,
                },
                event: {
                    id: 'event',
                    name: 'Event',
                    dataType: 'Enum',
                    required: true,
                    getValues: getEvents,
                },
            },
            async dynamicOverrides(properties) {
                const event = await getEvent(properties);
                const ports = Object.assign({}, this.ports);
                if (event && dataTypesRaxaToControlFlow[event.type]) {
                    ports.eventObj = propertyToValuePort('eventObj', event, 'ValueOutput');
                }
                return { ports };
            },
        },
        fireEvent: {
            id: 'fireEvent',
            name: 'Fire event',
            group: ['Raxa'],
            ports: {
                event: {
                    id: 'event',
                    kind: 'EventInput',
                },
            },
            properties: {
                interface: {
                    id: 'interface',
                    name: 'Interface',
                    dataType: 'Enum',
                    required: true,
                    getValues: getAllInterfaces,
                },
                event: {
                    id: 'event',
                    name: 'Event',
                    dataType: 'Enum',
                    required: true,
                    getValues: getEvents,
                },
            },
            async dynamicOverrides(properties) {
                const event = await getEvent(properties);
                const ports = Object.assign({}, this.ports);
                if (event && dataTypesRaxaToControlFlow[event.type]) {
                    ports.eventObj = propertyToValuePort('eventObj', event, 'ValueInput');
                }
                return { ports };
            },
        },
        getVariable: {
            id: 'getVariable',
            name: 'Get Variable',
            ports: {
                value: { id: 'value', kind: 'ValueOutput', dataType: 'Any' },
            },
            properties: {
                name: {
                    id: 'name',
                    name: 'Name',
                    dataType: 'String',
                    required: true,
                },
            },
        },
        setVariable: {
            id: 'setVariable',
            name: 'Set Variable',
            ports: {
                event: { id: 'event', kind: 'EventInput' },
                value: { id: 'value', kind: 'ValueInput', dataType: 'Any' },
            },
            properties: {
                name: {
                    id: 'name',
                    name: 'Name',
                    dataType: 'String',
                    required: true,
                },
            },
        },
        httpRequest: {
            id: 'httpRequest',
            name: 'Receive HTTP Request',
            group: ['Raxa'],
            ports: {
                event: { id: 'event', kind: 'EventOutput' },
                value: { id: 'value', kind: 'ValueOutput', dataType: 'String' },
            },
            properties: {
                name: {
                    id: 'name',
                    name: 'Name',
                    dataType: 'String',
                    required: true,
                },
            },
        },
        switch: {
            id: 'switch',
            name: 'Switch',
            group: ['Logic'],
            ports: {
                condition: {
                    id: 'condition',
                    kind: 'ValueInput',
                    name: 'condition',
                    dataType: 'Bool',
                },
                trueValue: {
                    id: 'trueValue',
                    kind: 'ValueInput',
                    name: 'true value',
                    dataType: 'Any',
                },
                falseValue: {
                    id: 'falseValue',
                    kind: 'ValueInput',
                    name: 'false value',
                    dataType: 'Any',
                },
                out: {
                    id: 'out',
                    kind: 'ValueOutput',
                    dataType: 'Any',
                },
            },
            properties: {},
            async dynamicOverrides(_, connections) {
                const trueValue = connections.get('trueValue')[0];
                const falseValue = connections.get('falseValue')[0];
                if (trueValue && falseValue) {
                    return {};
                }
                const ports = Object.assign({}, this.ports);
                ports.trueValue = Object.assign({}, this.ports.trueValue);
                ports.falseValue = Object.assign({}, this.ports.falseValue);
                ports.out = Object.assign({}, this.ports.out);
                if (trueValue && !falseValue) {
                    ports.trueValue.dataType = trueValue.dataType;
                    ports.falseValue.dataType = trueValue.dataType;
                    ports.out.dataType = trueValue.dataType;
                    if (trueValue.dataType === 'Enum') {
                        ports.trueValue.values = trueValue.values;
                        ports.falseValue.values = trueValue.values;
                        ports.out.values = trueValue.values;
                    }
                    else if (trueValue.dataType === 'Object') {
                        ports.trueValue.properties = trueValue.properties;
                        ports.falseValue.properties = trueValue.properties;
                        ports.out.properties = trueValue.properties;
                    }
                }
                else if (falseValue && !trueValue) {
                    ports.trueValue.dataType = falseValue.dataType;
                    ports.falseValue.dataType = falseValue.dataType;
                    ports.out.dataType = falseValue.dataType;
                    if (falseValue.dataType === 'Enum') {
                        ports.trueValue.values = falseValue.values;
                        ports.falseValue.values = falseValue.values;
                        ports.out.values = falseValue.values;
                    }
                    else if (falseValue.dataType === 'Object') {
                        ports.trueValue.properties = falseValue.properties;
                        ports.falseValue.properties = falseValue.properties;
                        ports.out.properties = falseValue.properties;
                    }
                }
                return { ports };
            },
        },
    });
    const nodeTypes = checkNodeTypes(Object.assign(Object.assign(Object.assign(Object.assign({}, logic_1.nodeTypes), math_1.nodeTypes), utilities_1.nodeTypes), raxaNodes));
    return nodeTypes;
};
exports.getNodeTypes = getNodeTypes;
//# sourceMappingURL=nodeTypes.js.map